@@ -5,21 +5,35 @@ use warnings;
 use 5.010;
 use parent qw/autobox/;
 
-our $VERSION = "1.019";
+our $VERSION = "1.020";
 
 =head1 NAME
 
 autobox::Transform - Autobox methods to transform Arrays and Hashes
 
+=head1 CONTEXT
+
+L<autobox> provides the ability to call methods on native types,
+e.g. strings, arrays, and hashes as if they were objects.
+
+L<autobox::Core> provides the basic methods for Perl core functions
+like C<uc>, C<map>, and C<grep>.
+
+This module, C<autobox::Transform>, provides higher level and more
+specific methods to transform and manipulate arrays and hashes and, in
+particular when the values are hashrefs or objects.
+
+
+
 =head1 SYNOPSIS
 
     use autobox::Core;  # uniq, sort, join, sum, etc.
     use autobox::Transform;
 
-=head2 Array Examples
+=head2 Arrays with hashrefs/objects
 
-    # $books and $authors below are arrayrefs with either objects (or
-    # hashrefs)
+    # $books and $authors below are arrayrefs with either objects or
+    # hashrefs (the call syntax is the same)
 
     $books->map_by("genre");
     $books->map_by(price_with_tax => [$tax_pct]);
@@ -57,6 +71,10 @@ autobox::Transform - Autobox methods to transform Arrays and Hashes
     #     "Fantasy" => [ $fantasy_book_1 ],
     # },
 
+
+=head2 Arrays
+
+    # Flatten arrayrefs-of-arrayrefs
     $authors->map_by("books") # ->books returns an arrayref
     # [ [ $book1, $book2 ], [ $book3 ] ]
     $authors->map_by("books")->flat;
@@ -69,7 +87,7 @@ autobox::Transform - Autobox methods to transform Arrays and Hashes
     @books->to_array;
 
 
-=head2 Hash Examples
+=head2 Hashes
 
     # Upper-case the genre name, and make the count say "n books"
     $genre_count->map_each(sub { uc( $_[0] ) => "$_ books" });
@@ -291,13 +309,42 @@ sub throw {
     croak($error);
 }
 
+# Normalize the two method calling styles for accessor + args:
+#   $acessor, $args_arrayref, $modifier
+# or
+#   $acessor_and_args_arrayref, $modifier
+sub _normalized_accessor_args_subref {
+    my ($accessor, $args, $subref) = @_;
+
+    # Note: unfortunately, this won't allow the $subref (modifier) to
+    # become an arrayref later on when we do many types of modifiers
+    # (string eq, qr regex match, sub call, arrayref in) for
+    # filtering.
+    #
+    # That has to happen after the deprecation has expired and the old
+    # syntax is removed.
+    if(ref($args) eq "CODE") {
+        $subref = $args; # Move down one step
+        $args = undef;
+    }
+    if(ref($accessor) eq "ARRAY") {
+        ($accessor, my @args) = @$accessor;
+        $args = \@args;
+    }
+
+    return ($accessor, $args, $subref);
+}
+
 
 
 =head2 Transforming lists of objects vs list of hashrefs
 
-map_by, grep_by etc are called the same way regardless of whether the
-list contains objects or hashrefs. The items in the list must all be
-either objects or hashrefs.
+C<map_by>, C<grep_by> etc. (all methods named *_by) work with arrays
+that contain hashrefs or objects.
+
+These methods are called the same way regardless of whether the array
+contains objects or hashrefs. The items in the list must all be either
+objects or hashrefs.
 
 If the array contains objects, a method is called on each object
 (possibly with the arguments provided).
@@ -305,6 +352,36 @@ If the array contains objects, a method is called on each object
 If the array contains hashrefs, the hash key is looked up on each
 item.
 
+=head3 Calling accessor methods with arguments
+
+    $array->grep_by($accessor, $args, $subref)
+    $books->grep_by("price_with_discount", [ 5.0 ], sub { $_ < 15.0 })
+
+Call the method $accessor on each object using the arguments in the
+$args arrayref like so:
+
+    $object->$accessor(@$args)
+
+=head3 Alternate syntax
+
+    $array->grep_by($accessor_and_args, $subref)
+
+If the $accessor_and_args is specified as a string, it's a simple
+lookup/method call.
+
+    # method call without args
+    $books->grep_by("price", sub { $_ < 15.0 })
+    # $_->price()
+
+If the $accessor_and_args is specified as an arrayref, the first item
+is the method name, and the rest are the arguments to the method.
+
+    # method call with args
+    $books->grep_by([ price_with_discount => 5.0 ], sub { $_ < 15.0 })
+    # $_->price_with_discount(5.0)
+
+This syntax is the future proof, preferred one.
+
 
 =head2 List and Scalar Context
 
@@ -324,11 +401,11 @@ context. E.g.
     );
 
     $self->my_method(
-        # Correct, convert the list to an arrayref
+        # Correct, convert the returned list to an arrayref
         books => [ $books->grep_by("is_published") ],
     );
     $self->my_method(
-        # Correct, ensure scalar context i.e. an array ref
+        # Correct, ensure scalar context to get an array ref
         books => scalar $books->grep_by("is_published"),
     );
 
@@ -349,7 +426,8 @@ package # hide from PAUSE
 
 use autobox::Core;
 
-
+*_normalized_accessor_args_subref
+    = \&autobox::Transform::_normalized_accessor_args_subref;
 
 sub __invoke_by {
     my $invoke = shift;
@@ -420,7 +498,7 @@ Or get the hash key value. Examples:
 
 sub map_by {
     my $array = shift;
-    my ($accessor, $args) = @_;
+    my ($accessor, $args) = _normalized_accessor_args_subref(@_);
     return __invoke_by("map", $array, $accessor, $args);
 }
 
@@ -470,7 +548,7 @@ L<autobox::Core>.
 
 sub grep_by {
     my $array = shift;
-    my ($accessor, $args, $grep_subref) = @_;
+    my ($accessor, $args, $grep_subref) = _normalized_accessor_args_subref(@_);
     $grep_subref //= sub { !! $_ };
     # grep_by $value, if passed the method value must match the value?
     return __invoke_by(
@@ -504,7 +582,7 @@ Examples:
 
 sub uniq_by {
     my $array = shift;
-    my ($accessor, $args) = @_;
+    my ($accessor, $args) = _normalized_accessor_args_subref(@_);
     return __invoke_by("uniq", $array, $accessor, $args);
 }
 
@@ -607,7 +685,7 @@ sub __core_group_by {
 
 sub group_by {
     my $array = shift;
-    my( $accessor, $args, $value_sub ) = @_;
+    my ($accessor, $args, $value_sub) = _normalized_accessor_args_subref(@_);
 
     $value_sub //= sub { $_ };
     ref($value_sub) eq "CODE"
@@ -636,7 +714,7 @@ for the "Sci-fi" key.
 
 sub group_by_count {
     my $array = shift;
-    my( $accessor, $args ) = @_;
+    my ($accessor, $args) = _normalized_accessor_args_subref(@_);
 
     my $value_sub = sub {
         my $count = shift // 0; return ++$count;
@@ -665,7 +743,7 @@ are collected under the Sci-fi key.
 
 sub group_by_array {
     my $array = shift;
-    my( $accessor, $args ) = @_;
+    my ($accessor, $args) = _normalized_accessor_args_subref(@_);
 
     my $value_sub = sub {
         my $array = shift // [];
@@ -755,6 +833,9 @@ package # hide from PAUSE
 
 use autobox::Core;
 
+*_normalized_accessor_args_subref
+    = \&autobox::Transform::_normalized_accessor_args_subref;
+
 
 
 sub key_value {
